#!/usr/bin/php

<?php

//////////////////////////////////////////////////////////////////////
// Script to send Growl notifications over the network from any
// PHP-enabled machine
//////////////////////////////////////////////////////////////////////
// $Revision: 34307 $
// $Date: 2011-07-09 15:51:17 +0100 (Sat, 09 Jul 2011) $
// $LastChangedBy: kasprzyk $
//////////////////////////////////////////////////////////////////////
// The Growl PHP class is by Tyler Hall, available from:
//  http://github.com/tylerhall/php-growl/
// For further details, see the Growl home page at:
//  http://growl.info/
//////////////////////////////////////////////////////////////////////

// Extract the command-line arguments
$cmds = getArguments( $argv, $argc );
if( $cmds === false ) displayHelp( $argv );

// Create a new instance of the Growl class
$growl = new Growl( $cmds['address'], $cmds['password'], 'Magma' );
$type = 'Command-Line Growl Notification';
$growl->addNotification( $type );
$growl->register();

// Send the message
$status = $growl->notify( $type, $cmds['title'], $cmds['message'] ) ? 0 : 1;
exit( $status );

//////////////////////////////////////////////////////////////////////
// Local functions
//////////////////////////////////////////////////////////////////////

// getArguments - Returns an associative array of the arguments, or false.
function getArguments( $argv, $argc )
{
    if( $argc != 5 ) return false;
    // Extract the data
    $cmds = array();
    $cmds['address'] = trim( $argv[1] );
    $cmds['password'] = trim( $argv[2] );
    $cmds['title'] = trim( $argv[3] );
    $cmds['message'] = trim( $argv[4] );
    // Check that the user wasn't trying to pass -args
    if( empty( $cmds['address'] )  || $cmds['address'][0] == '-'  ||
        empty( $cmds['password'] ) || $cmds['password'][0] == '-' ||
        empty( $cmds['title'] )    || $cmds['title'][0] == '-'    ||
        empty( $cmds['message'] )  || $cmds['message'][0] == '-' )
        return false;
    // Looks good; return the values
    return $cmds;
}

// displayHelp - Displays a help message and exits
function displayHelp( $argv )
{
    echo 'Usage: ' . basename( $argv[0] ) . " address password title message\n";
    exit( 1 );
}

//////////////////////////////////////////////////////////////////////
// Growl class
//////////////////////////////////////////////////////////////////////

class Growl
{
    const GROWL_PRIORITY_LOW = -2;
    const GROWL_PRIORITY_MODERATE = -1;
    const GROWL_PRIORITY_NORMAL = 0;
    const GROWL_PRIORITY_HIGH = 1;
    const GROWL_PRIORITY_EMERGENCY = 2;

    private $appName;
    private $address;
    private $notifications;
    private $password;
    private $port;

    public function __construct($address, $password = '', $app_name = 'PHP Growl')
    {
        $this->appName       = utf8_encode($app_name);
        $this->address       = $address;
        $this->notifications = array();
        $this->password      = $password;
        $this->port          = 9887;
    }

    public function addNotification($name, $enabled = true)
    {
        $this->notifications[] = array('name' => utf8_encode($name), 'enabled' => $enabled);
    }

    public function register()
    {
        $data         = '';
        $defaults     = '';
        $num_defaults = 0;

        for($i = 0; $i < count($this->notifications); $i++)
        {
            $data .= pack('n', strlen($this->notifications[$i]['name'])) . $this->notifications[$i]['name'];
            if($this->notifications[$i]['enabled'])
            {
                $defaults .= pack('c', $i);
                $num_defaults++;
            }
        }

        // pack(Protocol version, type, app name, number of notifications to register)
        $data  = pack('c2nc2', 1, 0, strlen($this->appName), count($this->notifications), $num_defaults) . $this->appName . $data . $defaults;
        $data .= pack('H32', md5($data . $this->password));

        return $this->send($data);
    }

    public function notify($name, $title, $message, $priority = 0, $sticky = false)
    {
        $name     = utf8_encode($name);
        $title    = utf8_encode($title);
        $message  = utf8_encode($message);
        $priority = intval($priority);

        $flags = ($priority & 7) * 2;
        if($priority < 0) $flags |= 8;
        if($sticky) $flags |= 256;

        // pack(protocol version, type, priority/sticky flags, notification name length, title length, message length. app name length)
        $data = pack('c2n5', 1, 1, $flags, strlen($name), strlen($title), strlen($message), strlen($this->appName));
        $data .= $name . $title . $message . $this->appName;
        $data .= pack('H32', md5($data . $this->password));

        return $this->send($data);
    }

    private function send($data)
    {
        if(function_exists('socket_create') && function_exists('socket_sendto'))
        {
            $sck = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
            socket_sendto($sck, $data, strlen($data), 0x100, $this->address, $this->port);
            return true;
        }
        elseif(function_exists('fsockopen'))
        {
            $fp = fsockopen('udp://' . $this->address, $this->port);
            fwrite($fp, $data);
            fclose($fp);
            return true;
        }

        return false;
    }
}
