freeze;

/////////////////////////////////////////////////////////////////////////
// utilities.m
/////////////////////////////////////////////////////////////////////////
// $Revision: 30736 $
// $Date: 2010-11-22 07:18:15 +1100 (Mon, 22 Nov 2010) $
// $LastChangedBy: kasprzyk $
/////////////////////////////////////////////////////////////////////////
// Author: Alexander Kasprzyk
/////////////////////////////////////////////////////////////////////////
// A collection of useful utilities.
/////////////////////////////////////////////////////////////////////////

// Escapes the characters ", `, \, and $ in the given string.
function escape_string(str)
    str:=SubstituteString(str,"\\","\\\\");
    str:=SubstituteString(str,"\"","\\\"");
    str:=SubstituteString(str,"`","\\\`");
    str:=SubstituteString(str,"$","\\\$");
    return str;
end function;

// Attempts to convert the string, starting at index 'startidx', to an integer.
// The first return value is true iff the entire string (starting at
// 'startidx') was successfully parsed. The second return value is the integer
// parsed so far. The third return value is the index of the first non-parsable
// character in the string. (If the string was successfully parsed through to
// the end, then the returned index will be zero.)
function is_string_integer( str, startidx )
    // Initialization
    idx:=IndexOfNonWhiteSpace(str,startidx);
    if idx eq 0 then return false,0,0; end if;
    numbers:=["0","1","2","3","4","5","6","7","8","9"];
    // Is there a negative sign?
    negative:=false;
    if str[idx] eq "-" then
        negative:=true;
        idx:=IndexOfNonWhiteSpace(str,idx+1);
    end if;
    // The next character had better be a number
    if idx eq 0 or not str[idx] in numbers then
        return false,0,startidx;
    end if;
    // Read the numbers
    val:=0;
    while idx le #str do
        num:=Index(numbers,str[idx]) - 1;
        if num eq -1 then
            break;
        end if;
        val *:= 10;
        val +:= num;
        idx +:= 1;
    end while;
    // Apply the negation
    if negative then val *:= -1; end if;
    // Check that what follows is white-space only
    if IndexOfNonWhiteSpace(str,idx) eq 0 then
        return true,val,0;
    else
        return false,val,idx;
    end if;
end function;

// The default path to curl.
function default_curl_path()
    curl:=Trim(GetEnv("CURL_PATH"));
    if #curl eq 0 then curl:="curl"; end if;
    return curl;
end function;

// Returns the hostname.
function host_name()
    return Split(Pipe("hostname",""))[1];
end function;

// Returns true if the given address looks like the localhost's address.
function is_localhost(address)
    // Is this easy?
    if address eq "127.0.0.1" or address eq "localhost" then
        return true;
    end if;
    // Resolve the given address if we can
    ip:=Trim(Pipe("host \"" cat address cat "\" 2>&1 || echo \"FAILED\"",""));
    if Index(ip,"FAILED") ne 0 then
        ip:=address;
    else
        idx:=Index(ip,"address");
        ip:=idx eq 0 select address else ip[idx + 8..#ip];
    end if;
    // Get the IP address(es) of the localhost
    ips:=["127.0.0.1"];
    listing:=Pipe("host `hostname` 2>&1 || echo \"FAILED\"","");
    if Index(listing,"FAILED") eq 0 then
        for res in Split(listing) do
            idx:=Index(res,"address");
            if idx ne 0 then
                Append(~ips,Trim(res[idx + 8..#res]));
            end if;
        end for;
    end if;
    // Is the ip address a local one?
    return ip in ips;
end function;
